//
//  AASAsynchronousFileLoader.h
//  AASFilesystemTools
//
//  Created by Andreas Schwarz on 3/11/08.
//  OS X 10.5 or later required.
/* Copyright (c) 2001-2009, Andreas Schwarz
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the names of the Andreas Schwarz or immaterial nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import <Cocoa/Cocoa.h>

@class AASFileItem;
@class AASFileFilter;
@class AASDirectoryFilter;
@class AASFileLoadOperation;
@class AASAsynchronousFileLoader;


/* AASAsynchronousFileLoader objects can be used to asynchronously get an array of AASFileItem (or subclass) objects for a given directory or directory tree. It will periodically call its delegate to update it with status information (the number of files found so far and most recent file found) and inform it when it is finished. Each AASAsynchronousFileLoader object is a one-use item; create a new one every time you wish to load files for a directory or directory tree. */


@interface NSObject (AASAsynchronousFileLoaderDelegate)

/* This is called periodically to give the delegate status updates. Implementing this method is optional. 'found' is useful for updating a status display. Not every file found will be passed in via 'file' (in fact, only a small percentage will) but it may also be useful for status updates. */
- (void)asynchronousFileLoader:(AASAsynchronousFileLoader *)loader updatedNumberOfFilesFound:(int)found latestFile:(AASFileItem *)file;

/* This method is required. It is called when the asynchronous loading of files has completed. 'files' is all file objects found (if any). If loading is cancelled via the -cancel method, this method will not be called. */
- (void)asynchronousFileLoader:(AASAsynchronousFileLoader *)loader finishedFindingFiles:(NSArray *)files;

@end



@interface AASAsynchronousFileLoader : NSObject {
	id delegate;
	
	AASFileFilter *fileFilter;
	AASDirectoryFilter *directoryFilter;
	AASFileLoadOperation *fileLoadOperation;
	
    NSTimer *updateTimer;
	
	NSLock *loadingArrayLock;
	NSArray *loadingArray;

	BOOL operationIsRecursive;
	BOOL operationIsRunning;
	BOOL operationCompleted;
}


/* Initializer */
- (id)initWithDelegate:(id)del;

/* Returns the delegate */
- (id)delegate;

/* Returns YES if the operation has begun (via one of the -start... methods) but hasn't finished yet. NO before it has begun or after it is finished. */
- (BOOL)isLoading;

/* Returns YES if the operation is finished loading files. */
- (BOOL)isComplete;

/* Returns the AASDirectoryFilter object being used to determine what is a directory and where its contents are. */
- (AASDirectoryFilter *)directoryFilter;

/* Sets an AASDirectoryFilter (or subclass) that this operation can use to determine what is a directory and where its contents are. Do not use after the -start method has been called. */
- (void)setDirectoryFilter:(AASDirectoryFilter *)dirFilter;

/* Returns the AASFileFilter object that is being used to filter files which are being added to the loading array, or nil if there is no filter. */
- (AASFileFilter *)fileFilter;

/* Sets an AASFileFilter (or subclass) object that allows the operation to filter out certain files. Do not use after one fo the 'start' methods has been called. */
- (void)setFileFilter:(AASFileFilter *)filter;


- (void)cancel;


- (void)startWithDirectory:(AASFileItem *)file;

/* Depth of -1 is infinite. Depth of 0 essentially just filters the files passed in without loading anything else. */
- (void)startRecursivelyWithFiles:(NSArray *)files toDepth:(CFIndex)depth;


@end
