//
//  AASFileItem.h
//
//  Created by Andreas Schwarz on Thu Apr 18 2002.
//  OS X 10.5 or later required.
/* Copyright (c) 2001-2009, Andreas Schwarz
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the names of the Andreas Schwarz or immaterial nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#import <Cocoa/Cocoa.h>
#import <Carbon/Carbon.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/param.h>




/* Keys for archive dictionaries */
extern NSString *AASArchiveDataKey;		// Will return an NSData object containing an AliasHandle
extern NSString *AASArchivePathKey;		// Will return the last known URL of the file



struct AASFileInfoList {
	id owner;
	struct timespec modTimespec;
	struct timespec changeTimespec;
	UInt8 *pathRepresentation;
	BOOL isValid;
	NSTimeInterval timeOfLastEvent;
	struct AASFileInfoList *nextItem;
};

struct AASFileChanges {
	BOOL fileValid;
	BOOL pathChanged;
	BOOL fileModified;
};



@interface AASFileItem : NSObject <NSCopying> {
	// Declared protected so that subclasses in this framework can have fast access to variables in specific instances.
	// DO NOT attempt to access these variables yourself
	@protected
    CFStringRef _path;
    CFURLRef _pathURL;
    FSRef _fsRef;
	
	struct AASFileInfoList *_pathInfo;
	
    unsigned int _deviceNumber;
    unsigned int _iNode;
    unsigned int _hashValue;
	
	
    BOOL refValid;
    NSDate *_modDate;
	    
    BOOL isBasicDirectory;
    BOOL isSymlink;
	
	@private
    NSMapTable *_container;
	
	int kernelQueue;
	int kernelQueueFileDescriptor;
}


/* Designated initializer. Do not use this to create a new instance (use newFileItemForURL: or equivalents, which will create a new instance and call the initializer only if necessary). Subclasses will want to implement this. ref, pathRep, and fileInfo may be NULL. */
- (id)initWithURL:(NSURL *)url fsRef:(FSRef *)ref pathRep:(UInt8 *)pathRep statInfo:(struct stat *)fileInfo;



/* Returns an AASFileItem object for the specified URL. If an instance already exists for that url, that will be reused (this is nice when you are using subclasses of AASFileItem that may contain a lot of information about a file; saves memory and time).  The returned value is not autoreleased; this works just like [[AASFileItem alloc] init], except for reusing instances when possible. */
+ (id)newFileItemForURL:(NSURL *)url;

/* Equivalent to -newFileItemForURL: but will make use of the FSRef if it becomes necessary to create a new instance (this is more efficient than -newFileItemForURL: or -newFileItemForFSRef: if you already have both handy). */
+ (id)newFileItemForURL:(NSURL *)url fsRef:(FSRef *)ref;

/* Equivalent to -newFileItemForURL: but can take a path. */
+ (id)newFileItemForPath:(NSString *)path;

/* Equivalent to -newFileItemForURL: but can take an FSRef pointer. Returns nil if the FSRef cannot be resolved. */
+ (id)newFileItemForFSRef:(FSRef *)ref;

/* Returns a the file associated with the archive (see -archiveDictionary) and (in the case of subclasses) may add persistent information stored in the archive to the file. Returns nil if file cannot be found. Subclasses should implement this to set persistent information as necessary (see -loadDataIntoArchive: for more). */
+ (id)newFileItemForArchiveDictionary:(NSDictionary *)archive;



/* Convenience class methods to convert to and from arrays of NSString paths / NSURLs / AliasHandle NSData objects to arrays of AASFileItem instances */
+ (NSArray *)fileItemsForPaths:(NSArray *)paths;
+ (NSArray *)pathsForFileItems:(NSArray *)files;
+ (NSArray *)fileItemsForURLs:(NSArray *)paths;
+ (NSArray *)urlsForFileItems:(NSArray *)files;
+ (NSArray *)fileItemsForArchiveDictionaries:(NSArray *)data;
+ (NSArray *)archiveDictionariesForFileItems:(NSArray *)files;




/* So they can be used as a key in an NSDictionary, file objects have a unique hash and respond to -isEqual: based on the associated file's device number and inode. */
- (unsigned int)hash;
- (id)copyWithZone:(NSZone *)zone;
- (unsigned int)fileDeviceNumber;
- (unsigned int)fileiNode;
- (void *)fileiNodeHash; // Hash based on inode only, returned in pointer form (useful as a key for NSMapTable)








/* Returns the last known path to the file, without checking if it has changed.  This is the most efficient method to get the path. Returns nil if the file cannot be found. */
- (NSString *)path;

/* Returns a copy of the last known path in the filesystem representation, without checking if it has changed.  This is the most efficient method to get the filesystem representation.  Returns NULL if the file cannot be found. If not NULL, the return value must be freed with free() when the caller is done with it. */
- (UInt8 *)filesystemRepresentation;

/* Returns the last known url to the file, without checking if it has changed.  This is the most efficient method to get the url. Returns nil if the file cannot be found. */
- (NSURL *)url;

/* Returns a pointer to an FSRef for the file (might be NULL if an FSRef could not be created or if the file cannot be found). */
- (FSRef *)fsRef;

/* Returns a dictionary containing data that can be saved to disk and will allow the file to be recreated later. */
- (NSDictionary *)archiveDictionary;


/* Called by the -archiveDictionary method to allow subclasses to add persistent information to the archive dictionary. Subclasses must call 'super.' Subclasses can make use of this information when overriding +newFileItemForArchiveDictionary: (For example, they can call 'super' and then set any necessary attributes to the returned file object before passing it along). Be aware the object you recieve in newFileItemForArchiveDictionary: may already have been in use, so you may wish to check that your values haven't been superceded by new information before setting them.  */
- (void)loadDataIntoArchive:(NSMutableDictionary *)dictionary;




/* Returns the current path of the file even if it has moved.  This is much, much more expensive than calling -(NSString *)path and really shouldn't be necessary unless you're not using an AASDirectorySubscription to track your files (usually through the AASFileManager instance). Returns nil if the file cannot be found. */
- (NSString *)currentPath;

/* Returns a copy of the current path in the filesystem representation of the file even if it has moved.  This is much, much more expensive than calling -(NSString *)path and really shouldn't be necessary unless you're not using an AASDirectorySubscription to track your files (usually through the AASFileManager instance).  Returns NULL if the file cannot be found. If not NULL, the return value must be freed with free() when the caller is done with it. */
- (UInt8 *)currentFilesystemRepresentation;

/* Returns the current URL of the file even if it has moved.  This is much, much more expensive than calling -url and really shouldn't be necessary unless you're not using an AASDirectorySubscription to track your files (usually through the AASFileManager instance). Returns nil if the file cannot be found. */
- (NSURL *)currentURL;



/* Returns the last known modification date for the file. */
- (NSDate *)modificationDate;

/* Returns the current modification date for the file even if it has changed since the file object was last updated. More expensive than just calling -modificationDate.  Returns nil if the file cannot be found. */
- (NSDate *)currentModificationDate;




/* Returns whether this file object is valid.  It is no longer valid if it points to a file that has been deleted or otherwise lost track of. In this case, -path, -url, -currentURL and related methods will also return nil or NULL. */
- (BOOL)isValid;

/* Returns whether this file is a directory.  This includes file packages and bundles, but does not include aliases or symlinks to directories. */
- (BOOL)isBasicDirectory;

/* Returns whether this file is a symbolic link. */
- (BOOL)isSymlink;



/* Permissions. Always checks the current state of the file */
- (BOOL)isReadable; // Traverses symlinks
- (BOOL)isWritable; // Traverses symlinks
- (BOOL)isDeletable;
- (BOOL)isExecutable; // Traverses symlinks
- (BOOL)isRenamable;
- (BOOL)isSeeable; // Parent folder is readable



// Observers and events

/* This is used by the -updateForObserver: method as well as by AASDirectorySubscription objects, but shouldn't be used by anyone else. */
- (struct AASFileChanges *)changesForObserver:(id)observer;

- (BOOL)removeFileObserver:(id)observer;

- (BOOL)registerFileObserver:(id)observer;

/* Called by the AASDirectorySubscription when the file has been changed in some way (the event dictionary is the same as what gets passed to subscribers to the AASDirectorySubscription).  Checks to see if the file has already updated for this event during this check cycle, and if not, calls updateForEvent: on itself.  See the updateForEvent: method's comments for more. */
- (void)eventOccurred:(NSDictionary *)event;

/* The updateForEvent: method is there in case subclassers want to implement some kind of automatic file-information updates whenever the file is known to change.  It is called on the object before any subscribers are sent the notification about the change in the file, so the information that is updated will be ready if the subscribers expect it.  For example, if the subclass includes information on the file's file size, when the file recieves an AASFileUpdatedEvent the subclass can check/update the file size value, so that later when subscribers get notified of the update, they can immediately access the newly updated file size information.  Also, if you are using the subclass in places without a AASDirectorySubscription (perhaps relying on periodic calls to the update method), you can override this in your subclass to notify any interested objects that the subclass instance has changed.  */
- (void)updateForEvent:(NSDictionary *)event;

/* This is useful in case you are using AASFileItem objects alone, without the AASDirectorySubscription.  When you call this, it will check to see if the file has changed/moved since the last time this method was called by that observer.  If it has changed, it will update itself through the updateForEvent: method and then return YES.  Returns NO if no changes have occurred.  Updates by this method won't confuse any AASDirectorySubscriptions that may be using this file object; the two update systems are separate and can be used together safely. */
- (BOOL)updateForObserver:(id)observer;




// Kernel Queues

/* Sets up the kernel queue. If this method isn't called, the file will poll for changes using lstat(). This method is used by the AASDirectorySubscription in order to use kernel queues rather than polling to watch for changes. */
- (void)initKQueue;

/* Stops the file object from using the kernel queue for notification of changes. Don't call this unless you are absolutely sure nobody else wants to use the kernel queue (ie. you're using file objects standalone without the AASFileManager's or AASDirectorySubscription's tracking functions). */
- (void)endKQueue;

/* Returns YES if the file object is using kernel queues to keep updated on status changes. */
- (BOOL)usingKQueue;


@end
