//
//  AASFileManagerOperations.h
//  AASFilesystemTools
//
//  Created by Andreas Schwarz on 3/3/08.
//  OS X 10.5 or later required.
/* Copyright (c) 2001-2009, Andreas Schwarz
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 
 Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 Neither the names of the Andreas Schwarz or immaterial nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import <Foundation/Foundation.h>
#import <CoreServices/CoreServices.h>


/* This AASFileManager category contains some convenient methods for getting file objects as well as methods to manipulate files associated with file objects. At the moment, no NSError object is returned; if a method returns NO there has been an unspecified error. */



// Error domain for NSError objects
extern NSString *AASFilesystemManagerErrorDomain;



/* Keys for file manipulation dictionaries */
extern NSString *AASOpSourceItemKey;				// Key for the AASBasicInfoFileItem for the source file of a file operation (the file you intend to move, copy, etc.)
extern NSString *AASOpDestinationItemKey;			// Key for the AASBasicInfoFileItem for the destination folder of a file operation (where you intend to move the source to, etc.)
extern NSString *AASOpNewNameKey;					// Key for the NSString that will be the new name of a file in a rename.
extern NSString *AASOpSIssueKey;					// After preparation, some source files may have been tagged with an issue using this key.
extern NSString *AASOpDIssueKey;					// After preparation, some destination files may have been tagged with an issue using this key.
extern NSString *AASOpResolutionKey;				// The solution the sender presents for the AASOpIssueKey is set with this key.

/* Values for file manipulation issues */
extern NSString *AASOpIssueInvalidFile;				// File is invalid (doesn't exist or wasn't given or destination wasn't a folder).
extern NSString *AASOpIssueNoPermission;			// The user does not have permission to perform the operation on the file.
extern NSString *AASOpIssueFileAlreadyExists;		// You are trying to move, copy, or rename a file to a folder where a file with the same name already exists. You may overwrite the file.
extern NSString *AASOpIssueCannotOverwrite;			// You are trying to move, copy, or rename a file to a folder where a file with the same name already exists, and you don't have permission to overwrite the file.
extern NSString *AASOpIssueRecursiveMove;			// You are trying to move a directory into one of its own descendants.

/* Values for file manipulation resolutions. You can only return one resolution so it must satisfy both source and destination issue if you get both for a file. */
extern NSString *AASOpResolutionSkip;				// Skip this file.
extern NSString *AASOpResolutionOverwrite;			// Overwrite the preexisting file with the new one you're moving.






@interface AASFileManager (FileOperations)


/* Convenience methods */

/* Method for getting an autoreleased array of file objects that represent the contents of a directory wihthout using an AASDirectorySubscription. If there's another object that has subscribed to the directory, this will be able to efficiently reuse that same collection. */
- (NSArray *)filesInDirectory:(AASFileItem *)directory;

/* Takes an array of AASFileItem objects and makes an array of dictionaries suitable for passing to the file manipulation functions. Pass nil to destination for no destination (only for moving files to trash) or pass an AASFileItem for the directory you want to move/copy/alias your files to. If you need to rename files or send your files to a number of different destinations, build the dictionaries manually rather than using this method. */
- (NSArray *)fileDictionariesForFileItems:(NSArray *)fileItems withDestination:(AASFileItem *)destination;






/* File Manipulation */







/* Takes an array of NSDictionary objects and prepares to move them to the trash. The returned array will be a modified copy of the input array, with an AASFileManipulationIssue key set for any files that need a decision by the sender. */
- (NSMutableArray *)prepareMoveFilesToTrash:(NSArray *)fileDicts sender:(id)sender;
/* Takes the modified output of prepareMoveFilesToTrash:, which should include the sender's responses to any AASFileManipulationIssue items, and moves the files to the trash by sending appleevents to the Finder */
- (BOOL)moveFilesToTrash:(NSArray *)fileDicts sender:(id)sender error:(NSError **)error;


- (NSMutableArray *)prepareToAliasFiles:(NSArray *)fileDicts sender:(id)sender;
- (BOOL)aliasFiles:(NSArray *)fileDicts sender:(id)sender error:(NSError **)error;


- (NSMutableArray *)prepareToLinkFiles:(NSArray *)fileDicts sender:(id)sender;
- (BOOL)linkFiles:(NSArray *)fileDicts sender:(id)sender error:(NSError **)error;


- (NSMutableArray *)prepareToMoveFiles:(NSArray *)fileDicts sender:(id)sender;
- (BOOL)moveFiles:(NSArray *)fileDicts sender:(id)sender error:(NSError **)error;

- (NSMutableArray *)prepareToCopyFiles:(NSArray *)fileDicts sender:(id)sender;
- (BOOL)copyFiles:(NSArray *)fileDicts sender:(id)sender error:(NSError **)error;


- (NSMutableArray *)prepareToRenameFiles:(NSArray *)fileDicts sender:(id)sender;
- (BOOL)renameFiles:(NSArray *)fileDicts sender:(id)sender error:(NSError **)error;




//- (BOOL)moveFile:(NSString *)file toDir:(NSString *)dir withSender:(id)sender;







@end
